import json


def elasticsearch_bulk_decorate(bulk_index, bulk_type, msg):
    """ Decorates the msg with elasticsearch bulk format and adds index and message type"""
    command = json.dumps({'index': {'_index': bulk_index, '_type': bulk_type}})
    return '{0}\n{1}\n'.format(command, msg)

def format_as_json(msg):
    return json.dumps(msg)

def format_as_elasticsearch_bulk_json(bulk_index, bulk_type, msg):
    payload = format_as_json(msg)
    return elasticsearch_bulk_decorate(bulk_index, bulk_type, payload)



from tagalog.shipper.statsd import StatsdShipper, get_from_msg


class StatsdTimerShipper(StatsdShipper):
    def __init__(self, metric, timed_field, host='127.0.0.1', port='8125'):
        self.timed_field = timed_field
        super(StatsdTimerShipper, self).__init__(metric, host, port)

    def _statsd_msg(self, msg):
        timed_value = get_from_msg(self.timed_field, msg)
        return '%f|ms' % timed_value



from __future__ import print_function, unicode_literals
import sys

from tagalog import io
from tagalog.filters import now


def main():
    for msg in io.lines(sys.stdin):
        print(now(), msg, end='')

if __name__ == '__main__':
    main()



import json

from tagalog.shipper.formatter import format_as_json, format_as_elasticsearch_bulk_json
from tagalog.shipper.ishipper import IShipper

class StdoutShipper(IShipper):
    def __init__(self, bulk=False, bulk_index='logs', bulk_type='message'):
        self.bulk = bulk
        self.bulk_index = bulk_index
        self.bulk_type = bulk_type


    def ship(self, msg):
        if self.bulk:
            payload = format_as_elasticsearch_bulk_json(self.bulk_index,self.bulk_type,msg)
        else:
            payload = format_as_json(msg)
        print(payload)



from __future__ import absolute_import
import os
import json
import logging

from itertools import chain
from redis import Connection, ConnectionError, RedisError, StrictRedis

from tagalog.shipper.ishipper import IShipper
from tagalog.shipper.formatter import format_as_elasticsearch_bulk_json, format_as_json
from tagalog.shipper.shipper_error import ShipperError
from tagalog._compat import urlparse, _xrange


log = logging.getLogger(__name__)

class RoundRobinConnectionPool(object):
    """
    Round-robin Redis connection pool
    """
    def __init__(self,
                 patterns=None,
                 max_connections_per_pattern=None,
                 connection_class=Connection):
        self.patterns = []
        self.num_patterns = 0
        self.pid = os.getpid()
        self.connection_class = connection_class
        self.max_connections_per_pattern = max_connections_per_pattern or 2 ** 31
        self._pattern_idx = 0
        self._created_connections = []
        self._available_connections = []
        self._in_use_connections = []

        if patterns is not None:
            for patt in patterns:
                self.add_pattern(patt)

    def _checkpid(self):
        if self.pid != os.getpid():
            self.disconnect()
            self.__init__(self.patterns,
                          self.max_connections_per_pattern,
                          self.connection_class)

    def _next_pattern(self):
        self._pattern_idx = (self._pattern_idx + 1) % self.num_patterns

    def add_pattern(self, pattern):
        self.patterns.append(pattern)
        self.num_patterns += 1
        self._created_connections.append(0)
        self._available_connections.append([])
        self._in_use_connections.append(set())

    def remove_pattern(self, pattern):
        idx = self.patterns.index(pattern)
        self.patterns.pop(idx)

        # Keep the pattern index pointing at the correct pattern
        if idx < self._pattern_idx:
            self._pattern_idx -= 1

        # Disconnect connections for the removed pattern
        conns = chain(self._available_connections[idx],
                      self._in_use_connections[idx])
        for conn in conns:
            conn.disconnect()

        # Relabel all remaining connections
        for c in self.all_connections():
            if c._pattern_idx > idx:
                c._pattern_idx -= 1

        self._created_connections.pop(idx)
        self._available_connections.pop(idx)
        self._in_use_connections.pop(idx)
        self.num_patterns -= 1

        # Final adjustment to the pattern index to ensure we're not pointing
        # past the end of the pattern list
        if self._pattern_idx > self.num_patterns - 1:
            self._pattern_idx = 0

    def all_connections(self):
        """Returns a generator over all current connection objects"""
        for i in _xrange(self.num_patterns):
            for c in self._available_connections[i]:
                yield c
            for c in self._in_use_connections[i]:
                yield c

    def get_connection(self, command_name, *keys, **options):
        """Get a connection from the pool"""
        self._checkpid()
        try:
            connection = self._available_connections[self._pattern_idx].pop()
        except IndexError:
            connection = self.make_connection()
        self._in_use_connections[self._pattern_idx].add(connection)
        self._next_pattern()
        return connection

    def make_connection(self):
        """Create a new connection"""
        if self._created_connections[self._pattern_idx] >= self.max_connections_per_pattern:
            raise ConnectionError("Too many connections")
        self._created_connections[self._pattern_idx] += 1
        conn = self.connection_class(**self.patterns[self._pattern_idx])
        conn._pattern_idx = self._pattern_idx
        return conn

    def release(self, connection):
        """Releases the connection back to the pool"""
        self._checkpid()
        if connection.pid == self.pid:
            idx = connection._pattern_idx
            self._in_use_connections[idx].remove(connection)
            self._available_connections[idx].append(connection)

    def purge(self, connection):
        """Remove the connection from rotation"""
        self._checkpid()
        if connection.pid == self.pid:
            idx = connection._pattern_idx
            if connection in self._in_use_connections[idx]:
                self._in_use_connections[idx].remove(connection)
            else:
                self._available_connections[idx].remove(connection)
            connection.disconnect()

    def disconnect(self):
        """Disconnect all connections in the pool"""
        for conn in self.all_connections():
            conn.disconnect()

class ResilientStrictRedis(StrictRedis):

    @property
    def execution_attempts(self):
        if not hasattr(self, '_execution_attempts'):
            self._execution_attempts = 1
        return self._execution_attempts

    @execution_attempts.setter
    def execution_attempts(self, num):
        self._execution_attempts = num

    def execute_command(self, *args, **options):
        """Execute a command and return a parsed response"""
        pool = self.connection_pool
        command_name = args[0]
        for i in _xrange(self.execution_attempts):
            connection = pool.get_connection(command_name, **options)
            try:
                connection.send_command(*args)
                res = self.parse_response(connection, command_name, **options)
                pool.release(connection)
                return res

            # If anything goes wrong in .send_command() or .parse_response(),
            # and we don't catch it, this connection will never be returned to
            # the pool, and thus will leak.
            #
            # So, catch *everything* here except SystemExits and
            # KeyboardInterrupts.
            except Exception:
                pool.purge(connection)
                if i >= self.execution_attempts - 1:
                    raise


class RedisShipper(IShipper):

    def __init__(self, urls, key='logs', bulk=False, bulk_index='logs', bulk_type='message'):
        self.urls = urls
        self.key = key
        self.bulk = bulk
        self.bulk_index = bulk_index
        self.bulk_type = bulk_type

        patts = [self._parse_url(u) for u in self.urls]
        self.pool = RoundRobinConnectionPool(patterns=patts)
        self.rc = ResilientStrictRedis(connection_pool=self.pool)
        self.rc.execution_attempts = self.pool.num_patterns

    def ship(self, msg):
        if self.bulk:
            payload = format_as_elasticsearch_bulk_json(self.bulk_index,self.bulk_type,msg)
        else:
            payload = format_as_json(msg)
        try:
            self.rc.lpush(self.key, payload)
        except Exception as e:
            log.warn('Could not ship message: {0}'.format(e))

    def _parse_url(self, url):
        parsed = urlparse(url)
        db = 0

        if parsed.path.startswith('/'):
            path = parsed.path[1:]
        else:
            path = parsed.path

        if path:
            try:
                db = int(path)
            except ValueError:
                msg = 'Could not parse "{0}" as a valid Redis DB number!'.format(path)
                raise ValueError(msg)

        return {'host': parsed.hostname or 'localhost',
                'port': parsed.port or 6379,
                'db': db}






import csv

from tagalog.shipper.redis import RedisShipper
from tagalog.shipper.stdout import StdoutShipper
from tagalog.shipper.statsd_counter import StatsdCounterShipper
from tagalog.shipper.statsd_timer import StatsdTimerShipper
from tagalog.shipper.ishipper import IShipper


SHIPPERS = {}


class NullShipper(IShipper):
    def __init__(self):
        pass

    def ship(self, msg):
        pass


def register_shipper(name, constructor):
    if name not in SHIPPERS:
        SHIPPERS[name] = constructor
    else:
        raise RuntimeError('Shipper "{0}" already defined!'.format(name))


def unregister_shipper(name):
    return SHIPPERS.pop(name, None)


def get_shipper(name):
    return SHIPPERS.get(name)


def str2bool(s):
    mapping = {'true': True,'false': False}
    return mapping[s]


def build_redis(*args, **kwargs):
    if 'bulk' in kwargs:
        kwargs['bulk'] = str2bool(kwargs['bulk'])
    return RedisShipper(urls=args,**kwargs)


def build_stdout(*args, **kwargs):
    if 'bulk' in kwargs:
        kwargs['bulk'] = str2bool(kwargs['bulk'])
    if args:
        raise ShipperError("unexpected positional arguments to stdout shipper")
    return StdoutShipper(**kwargs)


def build_statsd_counter(*args, **kwargs):
    if args:
        raise ShipperError("unexpected positional arguments to statsd shipper")
    return StatsdCounterShipper(**kwargs)


def build_statsd_timer(*args, **kwargs):
    if args:
        raise ShipperError("unexpected positional arguments to statsd_timer shipper")
    return StatsdTimerShipper(**kwargs)


def build_null(*args, **kwargs):
    return NullShipper()


register_shipper('redis', build_redis)
register_shipper('stdout', build_stdout)
register_shipper('statsd', build_statsd_counter) # to avoid breaking existing code
register_shipper('statsd_counter', build_statsd_counter)
register_shipper('statsd_timer', build_statsd_timer)
register_shipper('null', build_null)


def parse_shipper(description):
    clauses = next(csv.reader([description])) #reading only a single line
    kwargs = {}
    args = []
    for clause in clauses[1:]:
        if '=' in clause:
            key, val = clause.split("=")
            kwargs[key] = val
        else:
            args.append(clause)
    return clauses[0], args, kwargs


def build_shipper(description):
    """Takes a command-line description of a shipper and build the relevant shipper from it"""

    name, ship_args, kwargs = parse_shipper(description)

    return get_shipper(name)(*ship_args, **kwargs)



"""
The :mod:`filters` Module contains the definitions for the filters supported by
``logtag`` and ``logship``, as well as a number of functions used to construct
the filter chains.

A filter is a generator function which takes an iterable as its first argument,
and optionally takes additional configuration arguments. It returns a generator
yielding the filtered log lines.
"""

import csv
import datetime
import itertools
import json
import socket
import logging

FILTERS = {}

log = logging.getLogger(__name__)

class FilterError(Exception):
    pass


def pipeline(*functions):
    """
    Construct a filter pipeline from a list of filter functions

    Given a list of functions taking an iterable as their only argument, and
    which return a generator, this function will return a single function with the
    same signature, which applies each function in turn for each item in the
    iterable, yielding the results.

    That is, given filter functions ``a``, ``b``, and ``c``, ``pipeline(a, b,
    c)`` will return a function which yields ``c(b(a(x)))`` for each item ``x``
    in the iterable passed to it. For example:

    >>> def a(iterable):
    ...     for item in iterable:
    ...         yield item + ':a'
    ...
    >>> def b(iterable):
    ...     for item in iterable:
    ...         yield item + ':b'
    ...
    >>> pipe = pipeline(a, b)
    >>> data_in = ["foo", "bar"]
    >>> data_out = pipe(data_in)
    >>> [x for x in data_out]
    ['foo:a:b', 'bar:a:b']
    """
    head = functions[0]
    tail = functions[1:]
    if tail:
        def _fn(iterable):
            for i in pipeline(*tail)(head(iterable)):
                yield i
        return _fn
    else:
        return head


def build(description):
    """Build a filter chain from a filter description string"""
    filters = next(csv.reader([description]))
    filter_funcs = [get(f[0], f[1:]) for f in csv.reader(filters, delimiter=':')]
    return pipeline(*filter_funcs)


def get(name, args=[]):
    """
    Get a filter function from a filter name and a list of unparsed arguments
    """
    try:
        f = FILTERS[name]
    except KeyError:
        raise FilterError('No such filter: {0}'.format(name))

    f_args = []
    f_kwargs = {}

    for arg in args:
        a = arg.split('=', 1)
        if len(a) == 1:
            f_args.append(a[0])
        else:
            f_kwargs[a[0]] = a[1]

    # Construct the curried filter function
    def _filterfunc(iterable):
        for item in f(iterable, *f_args, **f_kwargs):
            yield item

    return _filterfunc


def init_txt(iterable):
    """
    Read lines of text from ``iterable`` and yield dicts with the line data
    stored in the ``@message`` field.

    >>> data_in = ["hello\\n", "world\\n"]
    >>> data_out = init_txt(data_in)
    >>> [x for x in data_out]
    [{'@message': 'hello'}, {'@message': 'world'}]
    """
    for line in iterable:
        txt = line.rstrip('\n')
        yield {'@message': txt}
FILTERS['init_txt'] = init_txt


def init_json(iterable):
    """
    Read lines of JSON text from ``iterable`` and parse each line as a JSON
    object. Yield dicts for each line that successfully parses as a JSON object.
    Unparseable events will be skipped and raise a warning.

    >>> data_in = ['{"@message": "one message"}', '{"@message": "another message"}']
    >>> data_out = init_json(data_in)
    >>> [x for x in data_out]
    [{u'@message': u'one message'}, {u'@message': u'another message'}]
    """
    for line in iterable:
        try:
            item = json.loads(line)
        except ValueError as e:
            log.warn('init_json: could not parse JSON message "{0}"'.format(line))
            log.warn('init_json: error was "{0}"'.format(e))
            continue

        if not isinstance(item, dict):
            log.warn('init_json: skipping message "{0}" (not a JSON object)'.format(line))
            continue

        yield item
FILTERS['init_json'] = init_json


def add_timestamp(iterable, override=False):
    """
    Compute an accurate timestamp for each item in ``iterable``, adding an
    accurate timestamp to each one when received. The timestamp is a
    usecond-precision ISO8601 string added to the ``@timestamp`` field.

    By default, existing ``@timestamp`` fields will not be overwritten. This
    behaviour can be toggled with the ``override`` argument.

    >>> data_in = [{'@message': 'one message'}, {'@message': 'another message'}]
    >>> data_out = add_timestamp(data_in)
    >>> [x for x in data_out]
    [{'@timestamp': '2013-05-13T10:37:56.766743Z', '@message': 'one message'},
     {'@timestamp': '2013-05-13T10:37:56.767185Z', '@message': 'another message'}]
    """
    k = '@timestamp'
    for item in iterable:
        if override or k not in item:
            item[k] = now()
        yield item
FILTERS['add_timestamp'] = add_timestamp


def add_source_host(iterable, override=False):
    """
    Add the FQDN of the current machine to the ``@source_host`` field of each
    item in ``iterable``.

    By default, existing ``@source_host`` fields will not be overwritten. This
    behaviour can be toggled with the ``override`` argument.

    >>> data_in = [{'@message': 'one message'}, {'@message': 'another message'}]
    >>> data_out = add_source_host(data_in)
    >>> [x for x in data_out]
    [{'@source_host': 'lynx.local', '@message': 'one message'},
     {'@source_host': 'lynx.local', '@message': 'another message'}]
    """
    k = '@source_host'
    source_host = socket.getfqdn()
    for item in iterable:
        if override or k not in item:
            item[k] = source_host
        yield item
FILTERS['add_source_host'] = add_source_host


def add_fields(iterable, **kw_fields):
    """
    Add fields to each item in ``iterable``. Each key=value pair provided is
    merged into the ``@fields`` object, which will be created if required.

    >>> data_in = [{'@message': 'one message'}, {'@message': 'another message'}]
    >>> data_out = add_fields(data_in, foo='bar', baz='qux')
    >>> [x for x in data_out]
    [{'@fields': {'foo': 'bar', 'baz': 'qux'}, '@message': 'one message'},
     {'@fields': {'foo': 'bar', 'baz': 'qux'}, '@message': 'another message'}]
    """
    k = '@fields'
    for item in iterable:
        if k not in item:
            item[k] = {}
        item[k].update(kw_fields)
        yield item
FILTERS['add_fields'] = add_fields


def add_tags(iterable, *taglist):
    """
    Add tags to each item in ``iterable``. Each tag is added to the ``@tags``
    array, which will be created if required.

    >>> data_in = [{'@message': 'one message'}, {'@message': 'another message'}]
    >>> data_out = add_tags(data_in, 'foo', 'bar')
    >>> [x for x in data_out]
    [{'@message': 'one message', '@tags': ['foo', 'bar']},
     {'@message': 'another message', '@tags': ['foo', 'bar']}]
    """
    k = '@tags'
    for item in iterable:
        try:
            item[k].extend(taglist)
        except (AttributeError, KeyError):
            item[k] = []
            item[k].extend(taglist)
        yield item
FILTERS['add_tags'] = add_tags


def parse_lograge(iterable):
    """
    Attempt to parse each dict or dict-like object in ``iterable`` as if it were
    in lograge format, (e.g. "status=200 path=/users/login time=125ms"), adding
    key-value pairs to '@fields' for each matching item.

    >>> data_in = [{'@message': 'path=/foo/bar status=200 time=0.060'},
    ...            {'@message': 'path=/baz/qux status=503 time=1.651'}]
    >>> data_out = parse_lograge(data_in)
    >>> [x for x in data_out]
    [{'@fields': {'status': '200', 'path': '/foo/bar', 'time': '0.060'}, '@message': 'path=/foo/bar status=200 time=0.060'},
     {'@fields': {'status': '503', 'path': '/baz/qux', 'time': '1.651'}, '@message': 'path=/baz/qux status=503 time=1.651'}]
    """
    for item in iterable:
        if '@message' not in item:
            log.warn('parse_lograge: skipping item missing "@message" key ("{0}")'.format(item))
            continue
        if '@fields' not in item:
            item['@fields'] = {}
        for kv in item['@message'].split():
            ret = kv.split('=', 1)
            if len(ret) == 2:
                item['@fields'][ret[0]] = ret[1]
        yield item
FILTERS['parse_lograge'] = parse_lograge


def now():
    return _now().strftime('%Y-%m-%dT%H:%M:%S.%fZ')


def _now():
    return datetime.datetime.utcnow()





class IShipper(object):
    """
    Abstract class representing a log shipper. Log shippers should implement
    the following methods:
    """

    def ship(self, message):
        raise NotImplementedError('IShipper subclasses should implement the "ship" method!')



from __future__ import print_function, unicode_literals
import json
import sys

from tagalog import io

def main():
    for line in io.lines(sys.stdin):
        msg = json.loads(line)
        if '@timestamp' in msg:
            print(msg['@timestamp'], msg['@message'])
        else:
            print(msg['@message'])

if __name__ == '__main__':
    main()



import logging
from os import environ as env

from . import logship
from . import logstamp
from . import logtag
from . import logtext

__all__ = ['logship', 'logstamp', 'logtag', 'logtext']

_log_level_default = logging.INFO
_log_level = getattr(logging, env.get('TAGALOG_LOGLEVEL', '').upper(), _log_level_default)

logging.basicConfig(format='%(levelname)s: %(message)s', level=_log_level)



from __future__ import print_function
import argparse
import csv
import json
import sys
import textwrap

from tagalog import io, filters

DEFAULT_FILTERS = 'init_txt,add_timestamp,add_source_host'

parser = argparse.ArgumentParser(description="Convert log data on STDIN to a "
                                             "stream of timestamped JSON "
                                             "documents on STDOUT.")
parser.add_argument('-f', '--filters', default=DEFAULT_FILTERS,
                    help='a list of filters to apply to each log line')
parser.add_argument('-a', '--filters-append', action='append',
                    help='A list of filters to apply to each log line '
                         '(appended to the default filter set)')


def main():
    args = parser.parse_args()
    filterlist = [args.filters]
    if args.filters_append:
        filterlist.extend(args.filters_append)
    pipeline = filters.build(','.join(filterlist))

    for msg in pipeline(io.lines(sys.stdin)):
        print(json.dumps(msg))


if __name__ == '__main__':
    main()



import os
import sys
from setuptools import setup, find_packages

from tagalog import __version__

requirements = ['redis==2.7.2']

if sys.version_info[:2] < (2, 7):
    requirements.append('argparse')

HERE = os.path.dirname(__file__)
try:
    long_description = open(os.path.join(HERE, 'README.rst')).read()
except:
    long_description = None

setup(
    name='tagalog',
    version=__version__,
    packages=find_packages(exclude=['test*']),
    include_package_data=True,

    # metadata for upload to PyPI
    author='Nick Stenning',
    author_email='nick@whiteink.com',
    url='https://github.com/nickstenning/tagalog',
    description='Tagalog: tools for manipulating, tagging and shipping log data',
    long_description=long_description,
    license='MIT',
    keywords='sysadmin log logging redis tail',

    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'logship=tagalog.command.logship:main',
            'logstamp=tagalog.command.logstamp:main',
            'logtag=tagalog.command.logtag:main',
            'logtext=tagalog.command.logtext:main'
        ]
    }
)



from __future__ import unicode_literals
import os
import sys

UTF8 = 'UTF-8'

BUF_DEFAULT = -1
BUF_UNBUFFERED = 0
BUF_LINEBUFFERED = 1


def lines(fp):
    """
    Read lines of UTF-8 from the file-like object given in ``fp``, making sure
    that when reading from STDIN, reads are at most line-buffered.

    UTF-8 decoding errors are handled silently. Invalid characters are
    replaced by U+FFFD REPLACEMENT CHARACTER.

    Line endings are normalised to newlines by Python's universal newlines
    feature.

    Returns an iterator yielding lines.
    """
    if fp.fileno() == sys.stdin.fileno():
        close = True

        try: # Python 3
            fp = open(fp.fileno(), mode='r', buffering=BUF_LINEBUFFERED, errors='replace')
            decode = False
        except TypeError:
            fp = os.fdopen(fp.fileno(), 'rU', BUF_LINEBUFFERED)
            decode = True

    else:
        close = False

        try:
            # only decode if the fp doesn't already have an encoding
            decode = (fp.encoding != UTF8)
        except AttributeError:
            # fp has been opened in binary mode
            decode = True

    try:
        while 1:
            l = fp.readline()
            if l:
                if decode:
                    l = l.decode(UTF8, 'replace')
                yield l
            else:
                break
    finally:
        if close:
            fp.close()




from __future__ import print_function, unicode_literals
import argparse
import json
import sys
import textwrap

from tagalog import io
from tagalog import filters
from tagalog.shipper import build_shipper

DEFAULT_FILTERS = 'init_txt,add_timestamp,add_source_host'

parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent("""
    Ship log data from STDIN to somewhere else, timestamping and preprocessing
    each log entry into a JSON document along the way."""),
    epilog='''Parameter that various shipper accept
    stdout: bulk: whether to output in elasticsearch bulk format (default: false)
            bulk_index: elasticsearch index name (default: logs)
            bulk_type: adds elasticsearch _type in json (default: message)
    redis:  redis,url1,url2,....
            url1,url2,..urln: multiple redis servers you want to ship to
            bulk,bulk_index,bulk_type: as stdout shipper above
            key: redis list name (default: logs)
    statsd: metric: string value eg %{var1}.literal.%{var2}, where var1 and var2 would
                    be replaced with values from the message (required parameter)
            host: statsd host (default: 127.0.0.1)
            port: statsd port (default: 8125)''')
parser.add_argument('-f', '--filters', default=DEFAULT_FILTERS,
                    help='A list of filters to apply to each log line')
parser.add_argument('-a', '--filters-append', action='append',
                    help='A list of filters to apply to each log line '
                         '(appended to the default filter set)')
parser.add_argument('-s', '--shipper', nargs='+', default='redis,redis://localhost:6379',
                    help='''Select the shipper to be used to ship logs from redis, statsd, stdout and null.
                            You can specify multiple shippers using space as a delimiter. Also other arguments
                            can be passed to shipper separated by commas.
                            eg -s statsd,metric=%%{source}.nginx.%%{status},host=statsd.cluster,port=34532''')



def main():
    args = parser.parse_args()

    shippers = []
    for shipper_desc in args.shipper:
        shippers.append(build_shipper(shipper_desc))

    filterlist = [args.filters]
    if args.filters_append:
        filterlist.extend(args.filters_append)
    pipeline = filters.build(','.join(filterlist))

    for msg in pipeline(io.lines(sys.stdin)):
        for shpr in shippers:
            shpr.ship(msg)

if __name__ == '__main__':
    main()



import re
import logging
import socket
import operator
from functools import reduce

from tagalog.shipper.ishipper import IShipper
from tagalog.shipper.shipper_error import ShipperError

log = logging.getLogger(__name__)


def get_from_msg(field, msg):
    if(field in msg):
        return msg[field]
    else:
        pieces = field.split('.')
        # fetch from nested dict
        return reduce(operator.getitem, pieces, msg)


class StatsdShipper(IShipper):
    def __init__(self, metric, host='127.0.0.1', port='8125'):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.metric = metric
        portnum = int(port)
        self.sock.connect((host, portnum))

    def ship(self, msg):
        try:
            real_msg = "%s:%s" % (self.__realised_metric(msg), self._statsd_msg(msg))
            self.sock.send(real_msg.encode('utf-8'))
        except socket.error as e:
            log.warn("Could not ship message via StatsdShipper: {0}".format(e))
        except KeyError as e:
            log.warn("Could not ship message via StatsdShipper: key {0} not found in message when constructing metric {1}".format(e,self.metric))

    def __realised_metric(self, msg):
        def replace_metric_field(match):
            field = match.group(1)
            value = get_from_msg(field,msg)
            return str(value)

        pattern = r'%{([^}]*)}'
        realised_metric = re.sub(pattern, replace_metric_field, self.metric)
        return realised_metric
